package sun.awt.windows;

import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.print.PrinterException;
import sun.print.ProxyGraphics2D;
import java.awt.image.SampleModel;
import java.awt.image.WritableRaster;
import java.awt.image.ColorModel;
import java.awt.image.MultiPixelPackedSampleModel;
import java.awt.image.ComponentSampleModel;
import sun.awt.image.BytePackedRaster;
import sun.awt.image.ByteComponentRaster;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.awt.geom.Rectangle2D;
import java.awt.Image;
import sun.font.PhysicalFont;
import java.util.Arrays;
import java.awt.font.GlyphVector;
import java.awt.font.TextLayout;
import java.awt.Color;
import java.awt.font.FontRenderContext;
import sun.font.Font2D;
import sun.font.TrueTypeFont;
import sun.font.CompositeFont;
import sun.font.FontUtilities;
import java.awt.Font;
import java.awt.geom.AffineTransform;
import java.awt.Stroke;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.BasicStroke;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterJob;
import java.awt.Graphics2D;
import sun.print.PathGraphics;

final class WPathGraphics extends PathGraphics
{
    private static final int DEFAULT_USER_RES = 72;
    private static final float MIN_DEVICE_LINEWIDTH = 1.2f;
    private static final float MAX_THINLINE_INCHES = 0.014f;
    private static boolean useGDITextLayout;
    private static boolean preferGDITextLayout;
    
    WPathGraphics(Graphics2D graphics, PrinterJob printerJob, Printable painter, PageFormat pageFormat, int pageIndex, boolean canRedraw) {
        super(graphics, printerJob, painter, pageFormat, pageIndex, canRedraw);
    }
    
    @Override
    public Graphics create() {
        return new WPathGraphics((Graphics2D)this.getDelegate().create(), this.getPrinterJob(), this.getPrintable(), this.getPageFormat(), this.getPageIndex(), this.canDoRedraws());
    }
    
    @Override
    public void draw(Shape shape) {
        Stroke stroke = this.getStroke();
        if ((stroke instanceof BasicStroke)) {
            BasicStroke stroke2 = null;
            BasicStroke stroke3 = (BasicStroke)stroke;
            float lineWidth = stroke3.getLineWidth();
            Point2D.Float float1 = new Point2D.Float(lineWidth, lineWidth);
            AffineTransform transform = this.getTransform();
            transform.deltaTransform(float1, float1);
            if (Math.min(Math.abs(float1.x), Math.abs(float1.y)) < 1.2f) {
                Point2D.Float float2 = new Point2D.Float(1.2f, 1.2f);
                try {
                    transform.createInverse().deltaTransform(float2, float2);
                    stroke2 = new BasicStroke(Math.max(Math.abs(float2.x), Math.abs(float2.y)), stroke3.getEndCap(), stroke3.getLineJoin(), stroke3.getMiterLimit(), stroke3.getDashArray(), stroke3.getDashPhase());
                    this.setStroke(stroke2);
                }
                catch (NoninvertibleTransformException ex) {}
            }
            super.draw(shape);
            if (stroke2 != null) {
                this.setStroke(stroke3);
            }
        }
        else {
            super.draw(shape);
        }
    }
    
    @Override
    public void drawString(String s, int n, int n2) {
        this.drawString(s, (float)n, (float)n2);
    }
    
    @Override
    public void drawString(String s, float n, float n2) {
        this.drawString(s, n, n2, this.getFont(), this.getFontRenderContext(), 0.0f);
    }
    
    @Override
    protected int platformFontCount(Font font, String s) {
        AffineTransform affineTransform = new AffineTransform(this.getTransform());
        affineTransform.concatenate(this.getFont().getTransform());
        int type = affineTransform.getType();
        if (type == 32 || (type & 0x40) != 0x0) {
            return 0;
        }
        Font2D font2D = FontUtilities.getFont2D(font);
        if (font2D instanceof CompositeFont || font2D instanceof TrueTypeFont) {
            return 1;
        }
        return 0;
    }
    
    private static boolean isXP() {
        String property = System.getProperty("os.version");
        return property != null && (float)Float.valueOf(property) >= 5.1f;
    }
    
    private boolean strNeedsTextLayout(String s, Font font) {
        char[] charArray = s.toCharArray();
        return FontUtilities.isComplexText(charArray, 0, charArray.length) && (!WPathGraphics.useGDITextLayout || (!WPathGraphics.preferGDITextLayout && (!isXP() || !FontUtilities.textLayoutIsCompatible(font))));
    }
    
    private int getAngle(Point2D.Double double1) {
        double degrees = Math.toDegrees(Math.atan2(double1.y, double1.x));
        if (degrees < 0.0) {
            degrees += 360.0;
        }
        if (degrees != 0.0) {
            degrees = 360.0 - degrees;
        }
        return (int)Math.round(degrees * 10.0);
    }
    
    private float getAwScale(double n, double n2) {
        float n3 = (float)(n / n2);
        if (n3 > 0.999f && n3 < 1.001f) {
            n3 = 1.0f;
        }
        return n3;
    }
    
    public void drawString(String str, float n, float n2, Font font, FontRenderContext fontRenderContext, float n3) {
        if (str.length() == 0) {
            return;
        }
        if (WPrinterJob.shapeTextProp) {
            super.drawString(str, n, n2, font, fontRenderContext, n3);
            return;
        }
        int strNeedsTextLayout = this.strNeedsTextLayout(str, font);
        if ((font.hasLayoutAttributes() || strNeedsTextLayout) && !this.printingGlyphVector) {
            new TextLayout(str, font, fontRenderContext).draw(this, n, n2);
            return;
        }
        if (!(strNeedsTextLayout == 0)) {
            super.drawString(str, n, n2, font, fontRenderContext, n3);
            return;
        }
        AffineTransform transform = this.getTransform();
        AffineTransform affineTransform = new AffineTransform(transform);
        affineTransform.concatenate(font.getTransform());
        int type = affineTransform.getType();
        int n4 = type != 32 && (type & 0x40) == 0x0;
        WPrinterJob wPrinterJob = (WPrinterJob)this.getPrinterJob();
        try {
            wPrinterJob.setTextColor((Color)this.getPaint());
        }
        catch (ClassCastException ex) {
            n4 = false;
        }
        if (n4 == 0) {
            super.drawString(str, n, n2, font, fontRenderContext, n3);
            return;
        }
        Point2D.Float o = new Point2D.Float(n, n2);
        Point2D.Float o2 = new Point2D.Float();
        if (font.isTransformed()) {
            AffineTransform transform2 = font.getTransform();
            float a = (float)transform2.getTranslateX();
            float a2 = (float)transform2.getTranslateY();
            if ((double)Math.abs(a) < 1.0E-5) {
                a = 0.0f;
            }
            if ((double)Math.abs(a2) < 1.0E-5) {
                a2 = 0.0f;
            }
            o.x += a;
            o.y += a2;
        }
        transform.transform(o, o2);
        if (this.getClip() != null) {
            this.deviceClip(this.getClip().getPathIterator(transform));
        }
        float size2D = font.getSize2D();
        double xRes = wPrinterJob.getXRes();
        double yRes = wPrinterJob.getYRes();
        double n5 = yRes / 72.0;
        int orientation = this.getPageFormat().getOrientation();
        if (orientation == 0 || orientation == 2) {
            xRes = (yRes = yRes);
        }
        affineTransform.scale(1.0 / (xRes / 72.0), 1.0 / (yRes / 72.0));
        Point2D.Double o3 = new Point2D.Double(0.0, 1.0);
        affineTransform.deltaTransform(o3, o3);
        double sqrt = Math.sqrt(o3.x * o3.x + o3.y * o3.y);
        float n6 = (float)((double)size2D * sqrt * n5);
        Point2D.Double o4 = new Point2D.Double(1.0, 0.0);
        affineTransform.deltaTransform(o4, o4);
        float awScale = this.getAwScale(Math.sqrt(o4.x * o4.x + o4.y * o4.y), sqrt);
        int angle = this.getAngle(o4);
        Point2D.Double o5 = new Point2D.Double(1.0, 0.0);
        transform.deltaTransform(o5, o5);
        double sqrt2 = Math.sqrt(o5.x * o5.x + o5.y * o5.y);
        Point2D.Double o6 = new Point2D.Double(0.0, 1.0);
        transform.deltaTransform(o6, o6);
        double sqrt3 = Math.sqrt(o6.x * o6.x + o6.y * o6.y);
        Font2D font2D = FontUtilities.getFont2D(font);
        if ((font2D instanceof TrueTypeFont)) {
            this.textOut(str, font, (TrueTypeFont)font2D, fontRenderContext, n6, angle, awScale, sqrt2, sqrt3, n, n2, o2.x, o2.y, n3);
        }
        else if ((font2D instanceof CompositeFont)) {
            CompositeFont compositeFont = (CompositeFont)font2D;
            float n7 = o2.x;
            float n8 = o2.y;
            char[] charArray = str.toCharArray();
            int length = charArray.length;
            int[] array = new int[length];
            compositeFont.getMapper().charsToGlyphs(length, charArray, array);
            for (int i = 0; i < length; n7 = o2.x, n8 = o2.y) {
                int offset = i;
                int slot;
                for (slot = array[offset] >>> 24; i < length && array[i] >>> 24 == slot; ++i) {}
                String str2 = new String(charArray, offset, i - offset);
                this.textOut(str2, font, compositeFont.getSlotFont(slot), fontRenderContext, n6, angle, awScale, sqrt2, sqrt3, n, n2, n7, n8, 0.0f);
                float n9 = (float)font.getStringBounds(str2, fontRenderContext).getWidth();
                n += n9;
                o.x += n9;
                transform.transform(o, o2);
            }
        }
        else {
            super.drawString(str, n, n2, font, fontRenderContext, n3);
        }
        return;
    }
    
    @Override
    protected boolean printGlyphVector(GlyphVector glyphVector, float x, float y) {
        if ((glyphVector.getLayoutFlags() & 0x1) != 0x0) {
            return false;
        }
        if (glyphVector.getNumGlyphs() == 0) {
            return true;
        }
        AffineTransform transform = this.getTransform();
        AffineTransform affineTransform = new AffineTransform(transform);
        Font font = glyphVector.getFont();
        affineTransform.concatenate(font.getTransform());
        int type = affineTransform.getType();
        int n = type != 32 && (type & 0x40) == 0x0;
        WPrinterJob wPrinterJob = (WPrinterJob)this.getPrinterJob();
        try {
            wPrinterJob.setTextColor((Color)this.getPaint());
        }
        catch (ClassCastException ex) {
            n = false;
        }
        if (WPrinterJob.shapeTextProp || n == 0) {
            return false;
        }
        Point2D.Float ptSrc = new Point2D.Float(x, y);
        Point2D glyphPosition = glyphVector.getGlyphPosition(0);
        ptSrc.x += (float)glyphPosition.getX();
        ptSrc.y += (float)glyphPosition.getY();
        Point2D.Float ptDst = new Point2D.Float();
        if (font.isTransformed()) {
            AffineTransform transform2 = font.getTransform();
            float a = (float)transform2.getTranslateX();
            float a2 = (float)transform2.getTranslateY();
            if ((double)Math.abs(a) < 1.0E-5) {
                a = 0.0f;
            }
            if ((double)Math.abs(a2) < 1.0E-5) {
                a2 = 0.0f;
            }
            ptSrc.x += a;
            ptSrc.y += a2;
        }
        transform.transform(ptSrc, ptDst);
        if (this.getClip() != null) {
            this.deviceClip(this.getClip().getPathIterator(transform));
        }
        float size2D = font.getSize2D();
        double xRes = wPrinterJob.getXRes();
        double yRes = wPrinterJob.getYRes();
        double n2 = yRes / 72.0;
        int orientation = this.getPageFormat().getOrientation();
        if (orientation == 0 || orientation == 2) {
            xRes = (yRes = yRes);
        }
        affineTransform.scale(1.0 / (xRes / 72.0), 1.0 / (yRes / 72.0));
        Point2D.Double o = new Point2D.Double(0.0, 1.0);
        affineTransform.deltaTransform(o, o);
        double sqrt = Math.sqrt(o.x * o.x + o.y * o.y);
        float n3 = (float)((double)size2D * sqrt * n2);
        Point2D.Double o2 = new Point2D.Double(1.0, 0.0);
        affineTransform.deltaTransform(o2, o2);
        float awScale = this.getAwScale(Math.sqrt(o2.x * o2.x + o2.y * o2.y), sqrt);
        int angle = this.getAngle(o2);
        Point2D.Double o3 = new Point2D.Double(1.0, 0.0);
        transform.deltaTransform(o3, o3);
        double sqrt2 = Math.sqrt(o3.x * o3.x + o3.y * o3.y);
        Point2D.Double o4 = new Point2D.Double(0.0, 1.0);
        transform.deltaTransform(o4, o4);
        double sqrt3 = Math.sqrt(o4.x * o4.x + o4.y * o4.y);
        int numGlyphs = glyphVector.getNumGlyphs();
        int[] glyphCodes = glyphVector.getGlyphCodes(0, numGlyphs, (int[])null);
        float[] glyphPositions = glyphVector.getGlyphPositions(0, numGlyphs, (float[])null);
        int n4 = 0;
        for (int i = 0; i < numGlyphs; ++i) {
            if ((glyphCodes[i] & 0xFFFF) >= 65534) {
                ++n4;
            }
        }
        if (n4 > 0) {
            int n5 = numGlyphs - n4;
            int[] array = new int[n5];
            float[] array2 = new float[n5 * 2];
            int n6 = 0;
            for (int j = 0; j < numGlyphs; ++j) {
                if ((glyphCodes[j] & 0xFFFF) < 65534) {
                    array[n6] = glyphCodes[j];
                    array2[n6 * 2] = glyphPositions[j * 2];
                    array2[n6 * 2 + 1] = glyphPositions[j * 2 + 1];
                    ++n6;
                }
            }
            numGlyphs = n5;
            glyphCodes = array;
            glyphPositions = array2;
        }
        AffineTransform scaleInstance = AffineTransform.getScaleInstance(sqrt2, sqrt3);
        float[] array3 = new float[glyphPositions.length];
        scaleInstance.transform(glyphPositions, 0, array3, 0, glyphPositions.length / 2);
        Font2D font2D = FontUtilities.getFont2D(font);
        if ((font2D instanceof TrueTypeFont)) {
            if (!wPrinterJob.setFont(font2D.getFamilyName(null), n3, font.getStyle() | font2D.getStyle(), angle, awScale)) {
                return false;
            }
            wPrinterJob.glyphsOut(glyphCodes, ptDst.x, ptDst.y, array3);
        }
        else if ((font2D instanceof CompositeFont)) {
            CompositeFont compositeFont = (CompositeFont)font2D;
            float n7 = ptDst.x;
            float n8 = ptDst.y;
            int k = 0;
            while (k < numGlyphs) {
                int from = k;
                int slot;
                for (slot = glyphCodes[from] >>> 24; k < numGlyphs && glyphCodes[k] >>> 24 == slot; ++k) {}
                PhysicalFont slotFont = compositeFont.getSlotFont(slot);
                if (!(slotFont instanceof TrueTypeFont)) {
                    return false;
                }
                if (!wPrinterJob.setFont(slotFont.getFamilyName(null), n3, font.getStyle() | slotFont.getStyle(), angle, awScale)) {
                    return false;
                }
                int[] copyOfRange = Arrays.copyOfRange(glyphCodes, from, k);
                float[] copyOfRange2 = Arrays.copyOfRange(array3, from * 2, k * 2);
                if (from != 0) {
                    Point2D.Float o5 = new Point2D.Float(x + glyphPositions[from * 2], y + glyphPositions[from * 2 + 1]);
                    transform.transform(o5, o5);
                    n7 = o5.x;
                    n8 = o5.y;
                }
                wPrinterJob.glyphsOut(copyOfRange, n7, n8, copyOfRange2);
            }
        }
        else {
            return false;
        }
        return true;
    }
    
    private void textOut(String removeControlChars, Font font, PhysicalFont physicalFont, FontRenderContext frc, float n, int n2, float n3, double sx, double sy, float n4, float n5, float n6, float n7, float n8) {
        String familyName = physicalFont.getFamilyName(null);
        int n9 = font.getStyle() | physicalFont.getStyle();
        WPrinterJob wPrinterJob = (WPrinterJob)this.getPrinterJob();
        if (!wPrinterJob.setFont(familyName, n, n9, n2, n3)) {
            super.drawString(removeControlChars, n4, n5, font, frc, n8);
            return;
        }
        float[] array = null;
        if (!this.okGDIMetrics(removeControlChars, font, frc, sx)) {
            removeControlChars = wPrinterJob.removeControlChars(removeControlChars);
            char[] charArray = removeControlChars.toCharArray();
            int length = charArray.length;
            GlyphVector glyphVector = null;
            if (!FontUtilities.isComplexText(charArray, 0, length)) {
                glyphVector = font.createGlyphVector(frc, removeControlChars);
            }
            if (glyphVector == null) {
                super.drawString(removeControlChars, n4, n5, font, frc, n8);
                return;
            }
            float[] glyphPositions = glyphVector.getGlyphPositions(0, length, (float[])null);
            glyphVector.getGlyphPosition(glyphVector.getNumGlyphs());
            AffineTransform scaleInstance = AffineTransform.getScaleInstance(sx, sy);
            float[] dstPts = new float[glyphPositions.length];
            scaleInstance.transform(glyphPositions, 0, dstPts, 0, glyphPositions.length / 2);
            array = dstPts;
        }
        wPrinterJob.textOut(removeControlChars, n6, n7, array);
        return;
    }
    
    private boolean okGDIMetrics(String str, Font font, FontRenderContext frc, double n) {
        double n2 = (double)Math.round(font.getStringBounds(str, frc).getWidth() * n);
        int gdiAdvance = ((WPrinterJob)this.getPrinterJob()).getGDIAdvance(str);
        if (n2 > 0.0 && gdiAdvance > 0) {
            double abs = Math.abs((double)gdiAdvance - n2);
            double n3 = (double)gdiAdvance / n2;
            if (n3 < 1.0) {
                n3 = 1.0 / n3;
            }
            return abs <= 1.0 || n3 < 1.01;
        }
        else {
            return true;
        }
    }
    
    @Override
    protected boolean drawImageToPlatform(Image img, AffineTransform affineTransform, Color white, int srcX, int srcY, int srcWidth, int srcHeight, boolean b) {
        BufferedImage bufferedImage = this.getBufferedImage(img);
        if (bufferedImage == null) {
            return true;
        }
        WPrinterJob wPrinterJob = (WPrinterJob)this.getPrinterJob();
        AffineTransform transform = this.getTransform();
        if (affineTransform == null) {
            affineTransform = new AffineTransform();
        }
        transform.concatenate(affineTransform);
        double[] array = new double[6];
        transform.getMatrix(array);
        Point2D.Float o = new Point2D.Float(1.0f, 0.0f);
        Point2D.Float o2 = new Point2D.Float(0.0f, 1.0f);
        transform.deltaTransform(o, o);
        transform.deltaTransform(o2, o2);
        Point2D.Float o3 = new Point2D.Float(0.0f, 0.0f);
        double distance = o.distance(o3);
        double distance2 = o2.distance(o3);
        double xRes = wPrinterJob.getXRes();
        double yRes = wPrinterJob.getYRes();
        double n = xRes / 72.0;
        double n2 = yRes / 72.0;
        if ((transform.getType() & 0x30) != 0x0) {
            if (distance > n) {
                distance = n;
            }
            if (distance2 > n2) {
                distance2 = n2;
            }
        }
        if (distance != 0.0 && distance2 != 0.0) {
            AffineTransform o4 = new AffineTransform(array[0] / distance, array[1] / distance2, array[2] / distance, array[3] / distance2, array[4] / distance, array[5] / distance2);
            Rectangle2D.Float o5 = new Rectangle2D.Float((float)srcX, (float)srcY, (float)srcWidth, (float)srcHeight);
            Rectangle2D bounds2D = o4.createTransformedShape(o5).getBounds2D();
            bounds2D.setRect(bounds2D.getX(), bounds2D.getY(), bounds2D.getWidth() + 0.001, bounds2D.getHeight() + 0.001);
            int n3 = (int)bounds2D.getWidth();
            int n4 = (int)bounds2D.getHeight();
            if (n3 > 0 && n4 > 0) {
                boolean n5 = true;
                if (!b && this.hasTransparentPixels(bufferedImage)) {
                    n5 = false;
                    if (this.isBitmaskTransparency(bufferedImage)) {
                        if (white == null) {
                            if (this.drawBitmaskImage(bufferedImage, affineTransform, white, srcX, srcY, srcWidth, srcHeight)) {
                                return true;
                            }
                        }
                        else if (white.getTransparency() == 1) {
                            n5 = true;
                        }
                    }
                    if (!this.canDoRedraws()) {
                        n5 = true;
                    }
                }
                else {
                    white = null;
                }
                if ((srcX + srcWidth > bufferedImage.getWidth(null) || srcY + srcHeight > bufferedImage.getHeight(null)) && this.canDoRedraws()) {
                    n5 = false;
                }
                if (n5 == 0) {
                    transform.getMatrix(array);
                    AffineTransform o7 = new AffineTransform(array[0] / n, array[1] / n2, array[2] / n, array[3] / n2, array[4] / n, array[5] / n2);
                    Rectangle2D bounds2D2 = transform.createTransformedShape(new Rectangle2D.Float((float)srcX, (float)srcY, (float)srcWidth, (float)srcHeight)).getBounds2D();
                    bounds2D2.setRect(bounds2D2.getX(), bounds2D2.getY(), bounds2D2.getWidth() + 0.001, bounds2D2.getHeight() + 0.001);
                    int n9 = (int)bounds2D2.getWidth();
                    int n10 = (int)bounds2D2.getHeight();
                    int n11 = n9 * n10 * 3;
                    int n12 = 8388608;
                    yRes = ((xRes < yRes) ? xRes : yRes);
                    int n13 = (int)yRes;
                    double n14 = 1.0;
                    double n15 = (double)n9 / (double)n3;
                    double n16 = (double)n10 / (double)n4;
                    n15 = ((n15 > n16) ? n16 : n15);
                    int n17 = (int)((double)n13 / n15);
                    if (n17 < 72) {
                        n17 = 72;
                    }
                    while (n11 > n12 && n13 > n17) {
                        n14 *= 2.0;
                        n13 /= 2;
                        n11 /= 4;
                    }
                    if (n13 < n17) {
                        n14 = yRes / (double)n17;
                    }
                    bounds2D2.setRect(bounds2D2.getX() / n14, bounds2D2.getY() / n14, bounds2D2.getWidth() / n14, bounds2D2.getHeight() / n14);
                    wPrinterJob.saveState(this.getTransform(), this.getClip(), bounds2D2, n14, n14);
                    return true;
                }
                else {
                    int n6 = 5;
                    Object cm = null;
                    ColorModel colorModel = bufferedImage.getColorModel();
                    int type = bufferedImage.getType();
                    if (colorModel instanceof IndexColorModel && colorModel.getPixelSize() <= 8 && (type == 12 || type == 13)) {
                        cm = (IndexColorModel)colorModel;
                        if ((n6 = type) == 12 && colorModel.getPixelSize() == 2) {
                            int[] array2 = new int[16];
                            cm.getRGBs(array2);
                            cm = new IndexColorModel(4, 16, array2, 0, cm.getTransparency() != 1, cm.getTransparentPixel(), 0);
                        }
                    }
                    int n7 = (int)bounds2D.getWidth();
                    int n8 = (int)bounds2D.getHeight();
                    if (true) {
                        if (cm == null) {
                            bufferedImage = new BufferedImage(n7, n8, n6);
                        }
                        else {
                            bufferedImage = new BufferedImage(n7, n8, n6, cm);
                        }
                        Graphics2D graphics = bufferedImage.createGraphics();
                        graphics.clipRect(0, 0, bufferedImage.getWidth(), bufferedImage.getHeight());
                        graphics.translate(-bounds2D.getX(), -bounds2D.getY());
                        graphics.transform(o4);
                        if (white == null) {
                            white = Color.white;
                        }
                        graphics.drawImage(bufferedImage, srcX, srcY, srcX + srcWidth, srcY + srcHeight, srcX, srcY, srcX + srcWidth, srcY + srcHeight, white, null);
                        graphics.dispose();
                    }
                    else {
                    }
                    Rectangle2D.Float o6 = new Rectangle2D.Float((float)(bounds2D.getX() * distance), (float)(bounds2D.getY() * distance2), (float)(bounds2D.getWidth() * distance), (float)(bounds2D.getHeight() * distance2));
                    WritableRaster raster = bufferedImage.getRaster();
                    byte[] array3;
                    if ((raster instanceof ByteComponentRaster)) {
                        array3 = ((ByteComponentRaster)raster).getDataStorage();
                    }
                    else if ((raster instanceof BytePackedRaster)) {
                        array3 = ((BytePackedRaster)raster).getDataStorage();
                    }
                    else {
                        return false;
                    }
                    int pixelBitStride = 24;
                    SampleModel sampleModel = bufferedImage.getSampleModel();
                    if ((sampleModel instanceof ComponentSampleModel)) {
                        pixelBitStride = ((ComponentSampleModel)sampleModel).getPixelStride() * 8;
                    }
                    else if ((sampleModel instanceof MultiPixelPackedSampleModel)) {
                        pixelBitStride = ((MultiPixelPackedSampleModel)sampleModel).getPixelBitStride();
                    }
                    else if (cm != null) {
                        int width = bufferedImage.getWidth();
                        int height = bufferedImage.getHeight();
                        if (width > 0 && height > 0) {
                            pixelBitStride = array3.length * 8 / width / height;
                        }
                    }
                    Shape clip = this.getClip();
                    this.clip(affineTransform.createTransformedShape(o5));
                    this.deviceClip(this.getClip().getPathIterator(this.getTransform()));
                    wPrinterJob.drawDIBImage(array3, o6.x, o6.y, (float)Math.rint((double)o6.width + 0.5), (float)Math.rint((double)o6.height + 0.5), 0.0f, 0.0f, (float)bufferedImage.getWidth(), (float)bufferedImage.getHeight(), pixelBitStride, cm);
                    this.setClip(clip);
                }
            }
        }
        return true;
    }
    
    @Override
    public void redrawRegion(Rectangle2D rectangle2D, double n, double n2, Shape shape, AffineTransform affineTransform) throws PrinterException {
        WPrinterJob printerJob = (WPrinterJob)this.getPrinterJob();
        Printable printable = this.getPrintable();
        PageFormat pageFormat = this.getPageFormat();
        int pageIndex = this.getPageIndex();
        BufferedImage bufferedImage = new BufferedImage((int)rectangle2D.getWidth(), (int)rectangle2D.getHeight(), 5);
        Graphics2D graphics = bufferedImage.createGraphics();
        ProxyGraphics2D proxyGraphics2D = new ProxyGraphics2D(graphics, printerJob);
        proxyGraphics2D.setColor(Color.white);
        proxyGraphics2D.fillRect(0, 0, bufferedImage.getWidth(), bufferedImage.getHeight());
        proxyGraphics2D.clipRect(0, 0, bufferedImage.getWidth(), bufferedImage.getHeight());
        proxyGraphics2D.translate(-rectangle2D.getX(), -rectangle2D.getY());
        proxyGraphics2D.scale((double)((float)(printerJob.getXRes() / n) / 72.0f), (double)((float)(printerJob.getYRes() / n2) / 72.0f));
        proxyGraphics2D.translate(-printerJob.getPhysicalPrintableX(pageFormat.getPaper()) / printerJob.getXRes() * 72.0, -printerJob.getPhysicalPrintableY(pageFormat.getPaper()) / printerJob.getYRes() * 72.0);
        proxyGraphics2D.transform(new AffineTransform(this.getPageFormat().getMatrix()));
        proxyGraphics2D.setPaint(Color.black);
        printable.print(proxyGraphics2D, pageFormat, pageIndex);
        graphics.dispose();
        if (shape != null) {
            this.deviceClip(shape.getPathIterator(affineTransform));
        }
        Rectangle2D.Float float1 = new Rectangle2D.Float((float)(rectangle2D.getX() * n), (float)(rectangle2D.getY() * n2), (float)(rectangle2D.getWidth() * n), (float)(rectangle2D.getHeight() * n2));
        printerJob.drawImage3ByteBGR(((ByteComponentRaster)bufferedImage.getRaster()).getDataStorage(), float1.x, float1.y, float1.width, float1.height, 0.0f, 0.0f, (float)bufferedImage.getWidth(), (float)bufferedImage.getHeight());
    }
    
    @Override
    protected void deviceFill(PathIterator pathIterator, Color color) {
        WPrinterJob wPrinterJob = (WPrinterJob)this.getPrinterJob();
        this.convertToWPath(pathIterator);
        wPrinterJob.selectSolidBrush(color);
        wPrinterJob.fillPath();
    }
    
    @Override
    protected void deviceClip(PathIterator pathIterator) {
        WPrinterJob wPrinterJob = (WPrinterJob)this.getPrinterJob();
        this.convertToWPath(pathIterator);
        wPrinterJob.selectClipPath();
    }
    
    @Override
    protected void deviceFrameRect(int n, int n2, int n3, int n4, Color color) {
        AffineTransform transform = this.getTransform();
        if ((transform.getType() & 0x30) != 0x0) {
            this.draw(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
            return;
        }
        Stroke stroke = this.getStroke();
        if ((stroke instanceof BasicStroke)) {
            BasicStroke basicStroke = (BasicStroke)stroke;
            int endCap = basicStroke.getEndCap();
            int lineJoin = basicStroke.getLineJoin();
            if (endCap == 2 && lineJoin == 0 && basicStroke.getMiterLimit() == 10.0f) {
                float lineWidth = basicStroke.getLineWidth();
                Point2D.Float float1 = new Point2D.Float(lineWidth, lineWidth);
                transform.deltaTransform(float1, float1);
                float min = Math.min(Math.abs(float1.x), Math.abs(float1.y));
                Point2D.Float float2 = new Point2D.Float((float)n, (float)n2);
                transform.transform(float2, float2);
                Point2D.Float float3 = new Point2D.Float((float)(n + n3), (float)(n2 + n4));
                transform.transform(float3, float3);
                float n5 = (float)(float3.getX() - float2.getX());
                float n6 = (float)(float3.getY() - float2.getY());
                WPrinterJob wPrinterJob = (WPrinterJob)this.getPrinterJob();
                if (wPrinterJob.selectStylePen(endCap, lineJoin, min, color)) {
                    wPrinterJob.frameRect((float)float2.getX(), (float)float2.getY(), n5, n6);
                }
                else if ((double)min / Math.min(wPrinterJob.getXRes(), wPrinterJob.getYRes()) < 0.014000000432133675) {
                    wPrinterJob.selectPen(min, color);
                    wPrinterJob.frameRect((float)float2.getX(), (float)float2.getY(), n5, n6);
                }
                else {
                    this.draw(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
                }
            }
            else {
                this.draw(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
            }
        }
        return;
    }
    
    @Override
    protected void deviceFillRect(int n, int n2, int n3, int n4, Color color) {
        AffineTransform transform = this.getTransform();
        if ((transform.getType() & 0x30) != 0x0) {
            this.fill(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
            return;
        }
        Point2D.Float float1 = new Point2D.Float((float)n, (float)n2);
        transform.transform(float1, float1);
        Point2D.Float float2 = new Point2D.Float((float)(n + n3), (float)(n2 + n4));
        transform.transform(float2, float2);
        ((WPrinterJob)this.getPrinterJob()).fillRect((float)float1.getX(), (float)float1.getY(), (float)(float2.getX() - float1.getX()), (float)(float2.getY() - float1.getY()), color);
        return;
    }
    
    @Override
    protected void deviceDrawLine(int n, int n2, int n3, int n4, Color color) {
        Stroke stroke = this.getStroke();
        if ((stroke instanceof BasicStroke)) {
            BasicStroke basicStroke = (BasicStroke)stroke;
            if (basicStroke.getDashArray() != null) {
                this.draw(new Line2D.Float((float)n, (float)n2, (float)n3, (float)n4));
                return;
            }
            float lineWidth = basicStroke.getLineWidth();
            Point2D.Float float1 = new Point2D.Float(lineWidth, lineWidth);
            AffineTransform transform = this.getTransform();
            transform.deltaTransform(float1, float1);
            float min = Math.min(Math.abs(float1.x), Math.abs(float1.y));
            Point2D.Float float2 = new Point2D.Float((float)n, (float)n2);
            transform.transform(float2, float2);
            Point2D.Float float3 = new Point2D.Float((float)n3, (float)n4);
            transform.transform(float3, float3);
            int endCap = basicStroke.getEndCap();
            int lineJoin = basicStroke.getLineJoin();
            if (float3.getX() == float2.getX() && float3.getY() == float2.getY()) {
                endCap = 1;
            }
            WPrinterJob wPrinterJob = (WPrinterJob)this.getPrinterJob();
            if (wPrinterJob.selectStylePen(endCap, lineJoin, min, color)) {
                wPrinterJob.moveTo((float)float2.getX(), (float)float2.getY());
                wPrinterJob.lineTo((float)float3.getX(), (float)float3.getY());
            }
            else {
                double min2 = Math.min(wPrinterJob.getXRes(), wPrinterJob.getYRes());
                if (endCap == 1 || ((n == n3 || n2 == n4) && (double)min / min2 < 0.014000000432133675)) {
                    wPrinterJob.selectPen(min, color);
                    wPrinterJob.moveTo((float)float2.getX(), (float)float2.getY());
                    wPrinterJob.lineTo((float)float3.getX(), (float)float3.getY());
                }
                else {
                    this.draw(new Line2D.Float((float)n, (float)n2, (float)n3, (float)n4));
                }
            }
        }
    }
    
    private void convertToWPath(PathIterator pathIterator) {
        float[] array = new float[6];
        WPrinterJob wPrinterJob = (WPrinterJob)this.getPrinterJob();
        int polyFillMode;
        if (pathIterator.getWindingRule() == 0) {
            polyFillMode = 1;
        }
        else {
            polyFillMode = 2;
        }
        wPrinterJob.setPolyFillMode(polyFillMode);
        wPrinterJob.beginPath();
        while (!pathIterator.isDone()) {
            switch (pathIterator.currentSegment(array)) {
                case 0: {
                    wPrinterJob.moveTo(array[0], array[1]);
                    break;
                }
                case 1: {
                    wPrinterJob.lineTo(array[0], array[1]);
                    break;
                }
                case 2: {
                    int penX = wPrinterJob.getPenX();
                    int penY = wPrinterJob.getPenY();
                    wPrinterJob.polyBezierTo((float)penX + (array[0] - (float)penX) * 2.0f / 3.0f, (float)penY + (array[1] - (float)penY) * 2.0f / 3.0f, array[2] - (array[2] - array[0]) * 2.0f / 3.0f, array[3] - (array[3] - array[1]) * 2.0f / 3.0f, array[2], array[3]);
                    break;
                }
                case 3: {
                    wPrinterJob.polyBezierTo(array[0], array[1], array[2], array[3], array[4], array[5]);
                    break;
                }
                case 4: {
                    wPrinterJob.closeFigure();
                    break;
                }
            }
            pathIterator.next();
        }
        wPrinterJob.endPath();
    }
    
    static {
        WPathGraphics.useGDITextLayout = true;
        WPathGraphics.preferGDITextLayout = false;
        String name = (String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.print.enableGDITextLayout"));
        if (name != null) {
            WPathGraphics.useGDITextLayout = Boolean.getBoolean(name);
            if (!WPathGraphics.useGDITextLayout && name.equalsIgnoreCase("prefer")) {
                WPathGraphics.useGDITextLayout = true;
                WPathGraphics.preferGDITextLayout = true;
            }
        }
    }
}
